@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Create network transition chord progressions
 
-our $VERSION = '0.0305';
+our $VERSION = '0.0306';
 
 use Carp qw(croak);
 use Data::Dumper::Compact qw(ddc);
@@ -57,6 +57,18 @@ has scale_note => (
 );
 
 
+has scale => (
+    is        => 'lazy',
+    init_args => undef,
+);
+
+sub _build_scale {
+    my ($self) = @_;
+    my @scale = get_scale_notes($self->scale_note, $self->scale_name);
+    return \@scale;
+}
+
+
 has octave => (
     is      => 'ro',
     isa     => sub { die "$_[0] is not a valid octave" unless $_[0] =~ /^-?\d+$/ },
@@ -100,11 +112,21 @@ has flat => (
 
 
 has graph => (
-    is      => 'ro',
-    isa     => sub { die "$_[0] is not a valid graph" unless ref($_[0]) =~ /^Graph/ },
-    default => sub { Graph::Directed->new },
+    is        => 'lazy',
+    init_args => undef,
 );
 
+sub _build_graph {
+  my ($self) = @_;
+    my $g = Graph::Directed->new;
+    for my $posn (keys %{ $self->net }) {
+        for my $p (@{ $self->net->{$posn} }) {
+            $g->add_edge($posn, $p);
+        }
+    }
+    return $g;
+}
+
 
 has verbose => (
     is      => 'ro',
@@ -119,49 +141,17 @@ sub generate {
     croak 'chords length must equal number of net keys'
         unless @{ $self->chords } == keys %{ $self->net };
 
-    # Build the graph
-    for my $posn (keys %{ $self->net }) {
-        for my $p (@{ $self->net->{$posn} }) {
-            $self->graph->add_edge($posn, $p);
-        }
-    }
     print 'Graph: ' . $self->graph, "\n" if $self->verbose;
 
     # Create a random progression
     my @progression;
-    my $v;
+    my $v; # Vertex
     for my $n (1 .. $self->max) {
-        if ($n == 1) {
-            if ($self->tonic == 0) {
-                $v = $self->graph->random_successor(1);
-            }
-            elsif ($self->tonic == 1) {
-                $v = 1;
-            }
-            else {
-                $v = $self->_full_keys;
-            }
-        }
-        elsif ($n == $self->max) {
-            if ($self->resolve == 0) {
-                $v = $self->graph->random_successor(scalar keys %{ $self->net });
-            }
-            elsif ($self->resolve == 1) {
-                $v = 1;
-            }
-            else {
-                $v = $self->_full_keys;
-            }
-        }
-        else {
-            $v = $self->graph->random_successor($v);
-        }
+        $v = $self->_next_successor($n, $v);
         push @progression, $v;
     }
     print "Progression: @progression\n" if $self->verbose;
 
-    my @scale = get_scale_notes($self->scale_note, $self->scale_name);
-
     my @chords = @{ $self->chords };
 
     if ($self->substitute) {
@@ -169,14 +159,14 @@ sub generate {
         for my $chord (@chords) {
             my $substitute = $self->sub_cond->() ? $self->substitution($chord) : $chord;
             if ($substitute eq $chord && $i < @progression && $self->sub_cond->()) {
-                $progression[$i] .= 't';
+                $progression[$i] .= 't'; # Indicate that we should tritone substitute
             }
             $chord = $substitute;
             $i++;
         }
     }
 
-    my @phrase = map { $self->_tt_sub(\@scale, \@chords, $_) } @progression;
+    my @phrase = map { $self->_tt_sub(\@chords, $_) } @progression;
     print "Phrase: @phrase\n" if $self->verbose;
 
     # Add octaves to the chord notes
@@ -207,6 +197,40 @@ sub generate {
     return \@notes;
 }
 
+sub _next_successor {
+    my ($self, $n, $v) = @_;
+
+    my $s;
+
+    if ($n == 1) {
+        if ($self->tonic == 0) {
+            $s = $self->graph->random_successor(1);
+        }
+        elsif ($self->tonic == 1) {
+            $s = 1;
+        }
+        else {
+            $s = $self->_full_keys;
+        }
+    }
+    elsif ($n == $self->max) {
+        if ($self->resolve == 0) {
+            $s = $self->graph->random_successor(scalar keys %{ $self->net });
+        }
+        elsif ($self->resolve == 1) {
+            $s = 1;
+        }
+        else {
+            $s = $self->_full_keys;
+        }
+    }
+    else {
+        $s = $self->graph->random_successor($v);
+    }
+
+    return $s;
+}
+
 sub _full_keys {
     my ($self) = @_;
     my @keys = grep { keys @{ $self->net->{$_} } > 0 } keys %{ $self->net };
@@ -214,7 +238,7 @@ sub _full_keys {
 }
 
 sub _tt_sub {
-    my ($self, $scale, $chords, $n) = @_;
+    my ($self, $chords, $n) = @_;
 
     my $note;
 
@@ -225,11 +249,11 @@ sub _tt_sub {
         my %stritone = map { $snotes[$_] => $snotes[($_ + 6) % @snotes] } 0 .. $#snotes;
 
         $n =~ s/t//;
-        $note = $ftritone{ $scale->[$n - 1] } || $stritone{ $scale->[$n - 1] };
-        print "Tritone: $scale->[$n - 1] => $note\n" if $self->verbose;
+        $note = $ftritone{ $self->scale->[$n - 1] } || $stritone{ $self->scale->[$n - 1] };
+        print "Tritone: $self->scale->[$n - 1] => $note\n" if $self->verbose;
     }
     else {
-        $note = $scale->[$n - 1];
+        $note = $self->scale->[$n - 1];
     }
 
     return $note . $chords->[$n - 1];
@@ -284,7 +308,7 @@ Music::Chord::Progression - Create network transition chord progressions
 
 =head1 VERSION
 
-version 0.0305
+version 0.0306
 
 =head1 SYNOPSIS
 
@@ -385,6 +409,12 @@ The (uppercase) name of the scale starting note.
 
 Default: C<C>
 
+=head2 scale
+
+The scale notes.  This is a computed attribute.
+
+Default: C<[C D E F G A B]>
+
 =head2 octave
 
 The octave number of the scale.
@@ -444,7 +474,7 @@ Default: C<0>
 
 =head2 graph
 
-The network transition L<Graph> object.
+The network transition L<Graph> object.  This is a computed attribute.
 
 Default: C<Graph::Directed>
 
